
validate(){
	if [ $ITER -eq ' ' ]; then
		ITER=20
	fi
	if [ $INC -eq ' ' ]; then
		INC=50
	fi
	if [ $SAMPLES -eq ' ' ]; then
		SAMPLES=200
	fi
}

generate_ls_ts()
{
 echo "First - Generating LS and TS's"
 echo ""
 # The test set contain always 2000 strings
 ./m-sets -i ../digits/r64-all -sfg 2000 -fg ../tmp/test-r64.txt -sg ../tmp/allMtest.txt -vs 8

 # The learn sets contain initially 200 strings, by default
 # and by each iteration its incremented in 50 strings
 COUNT=1
 while [ $COUNT -le $ITER ]; do
	INCREMENT=$[$COUNT*$INC+$SAMPLES]
        ./m-sets -i ../digits/r64-e00 -sfg $INCREMENT -fg ../tmp/learn-r64_$COUNT.txt -vs 8
        let COUNT=COUNT+1
 done
}

generate_pairs()
{
 echo "Second - Generating the Databases"
 echo ""
 COUNT=1
 while [ $COUNT -le $ITER ]; do
	./m-nn -i ../tmp/learn-r64_$COUNT.txt -ed ../tmp/ED-cost.txt -o ../tmp/db-r64_$COUNT.txt
	let COUNT=COUNT+1
 done
}

learn_c_transducer()
{
 echo "Third - Learning the Conditional Transducer"
 echo ""
 COUNT=1
 while [ $COUNT -le $ITER ]; do
       	./lsed -i ../tmp/db-r64_$COUNT.txt -o ../tmp/ctrans-r64_$COUNT.txt -na c
	let COUNT=COUNT+1
 done
}

learn_j_transducer()
{
 echo "Fourth - Learning the Join Transducer"
 echo ""
 COUNT=1
 while [ $COUNT -le $ITER ]; do
       ./lsed -i ../tmp/db-r64_$COUNT.txt -o ../tmp/jtrans-r64_$COUNT.txt -na j
       let COUNT=COUNT+1
 done
}

classifier()
{
 echo "Fifth - Evaluating the Algorithms"
 echo ""
 COUNT=1
 while [ $COUNT -le $ITER ]; do
	./classifier -il ../tmp/learn-r64_$COUNT.txt -it ../tmp/test-r64.txt \
	-i ../tmp/ctrans-r64_$COUNT.txt -e ../tmp/ED-cost.txt >> ../tmp/results-$COUNT.txt
    let COUNT=COUNT+1
 done
}

allInOne()
{
 # The test set contain always 2000 strings
 ./m-sets -i ../digits/r64-e00 -sfg 2000 -fg ../tmp/test-r64.txt -sg ../tmp/allMtest.txt -vs 8
 COUNT=1
 while [ $COUNT -le $ITER ]; do
	# The learn sets contain initially 200 strings
	./m-sets -i ../tmp/allMtest.txt -sfg $COUNT -fg ../tmp/learn-r64_$COUNT.txt -vs 8

	./m-nn -i ../tmp/learn-r64_$COUNT.txt -ed ../tmp/ED-cost.txt -o ../tmp/db-r64_$COUNT.txt

      	./lsed -i ../tmp/db-r64_$COUNT.txt -o ../tmp/ctrans-r64_$COUNT.txt -na c

       	./lsed -i ../tmp/db-r64_$COUNT.txt -o ../tmp/jtrans-r64_$COUNT.txt -na j

	./classifier -il ../tmp/learn-r64_$COUNT.txt -it ../tmp/test-r64.txt -ij ../tmp/jtrans-r64_$COUNT.txt \
	-t ../tmp/ctrans-r64_$COUNT.txt -e ../tmp/ED-cost.txt >> ../tmp/results-$COUNT.txt

 	let COUNT=COUNT+1
 done
}

        OPTION="$1"
	ITER="$2"
	INC="$3"
	SAMPLES="$4"
	case $OPTION in
		-a)	validate
			allInOne
		;;
		-s)	validate
			generate_ls_ts
			generate_pairs
			learn_c_transducer
			learn_j_transducer
			classifier
		;;
		-c)	validate
			classifier
		;;
		*) echo "Usage: ./character_hw.sh -[a:all|s:step to step|c: classifier]"
		   echo " [numOfIterations] [Increment] [InitialSamples]"
		exit 1
		;;
	esac
