#include <iostream>
#include <fstream>
#include "classifier.h"

using namespace std;

//================= Main =========================================

int main (int argc, char *argv[]) {
	int i, argcount=0;
	Pair_database db_ls, db_ts;
	Transducer trans_tm;
	Algorithm alg;
	Edit_distance ed;
	string file_ts; 	//String pairs file
	string file_ls; 	//Learning string file 
	string file_tm;		//Edit distance file 
	string file_ed;		//Conditional file 
	int verbosity = 0;
	int trans_model = 0; 	// conditional : 0 , joint : 1

	//To analize the input arguments
	for (i = 1; i < argc; i++) {
		if (strcmp("-il", argv[i]) == 0) {
			argcount++;
			file_ls = argv[++i];
		} else if (strcmp("-it", argv[i]) == 0) {
			argcount++;
			file_ts = argv[++i];
		} 
		 else if (strcmp("-e", argv[i]) == 0) {
			argcount++;
			file_ed = argv[++i];
		} else if (strcmp("-i", argv[i]) == 0) {
			argcount++;
			file_tm = argv[++i];
		} else if (strcmp ("-v", argv[i]) == 0) {
			verbosity = atoi(argv[++i]);
		} else if (strcmp ("-m", argv[i]) == 0) {
			trans_model = atoi(argv[++i]);
		} else if (strcmp ("-H", argv[i]) == 0) {
			cout << "NAME" << endl;
			cout << "	classifier : This make up the classification of test" <<
				" samples with classical edit distance and " << endl;
			cout << "		     some stochastic transducer model. Furthermore," <<
				" this module gives the accuracy" << endl;
			cout << "		     of this implementation." << endl;
			cout << "SYNOPSIS" << endl;
			cout << "	classifier -il {learn-file} -it {test-file} -e {edit-distance-costs}" <<
				" -i {transducer-model}" << endl;
			cout << "DESCRIPTION" << endl;
			cout << "	This module make up a classification procedure and give the accuracy of it" << endl;
			cout << "	Mandatory arguments:" << endl;
			cout << "	-il	Input file with the representation strings of some entity" << endl;
			cout << "	-il	Input file with the representation strings of some entity" << endl;
			cout << "	-e	Output file where the stochastic transducer model will be saved" << endl;
			cout << "	-i	Output file where the stochastic transducer model will be saved" << endl;
			cout << "	Optional arguments" << endl;
			cout << "	-m {c,j}	Normalization mode where c is conditional and j is joint" << endl;
			cout << "			(default c)" << endl;
			cout << "	-v verbosity level" << endl;
			cout << "	-h brief help" << endl;
			cout << "	-H help" << endl;
			cout << "FILE FORMAT" << endl;
			cout << "	Input File: Test and Learn file." << endl;
			cout << endl;
			cout << "	VocabularySize	StringLength(default 0)" << endl;
			cout << "	Class	RepresentationString" << endl;
			cout << endl;
			cout << "	Edit Distance and Transducer model file:" << endl;
			cout << endl;
			cout << "	VocabularySize" << endl;
			cout << "	CostsMatrix" << endl;
			cout << endl;
			cout << "	Output:" << endl;
			cout << endl;
			cout << "	NameAlgorithm	LearnSize TestSize	TruePositive	Accuracy" <<
				"	(SecondAlgorithm) ..." << endl;
			cout << endl;
			cout << "SEE ALSO" << endl;
			cout << "	______________" << endl;
			exit(0);
		} else if (strcmp ("-h", argv[i]) == 0) {
			cout << "use " << argv[0] << " [options]" << endl;
			cout << "Mandatory :" << endl;
			cout << "  -il   <string>	Learning strings file" << endl;
			cout << "  -it   <string>	Testing strings file" << endl;
			cout << "  -e    <string>	Edit distance costs file" << endl;
			cout << "  -i    <string>	Transducer model file" << endl;
			cout << "Optional :" << endl;
			cout << "  -v    <int> 		Verbosity level" << endl;
			cout << "  -m	 {c,j}		Type of transducer model (default c:conditional)" << endl;
			cout << "  -h 			Print brief help" << endl;
			cout << "  -H 			Print help" << endl;
			exit(0);
		} else {
			cout << "ERROR: unknown option " << argv[i] << endl;
			exit(-1);
		}
	}

	if (argcount < 4) {
		cout << "ERROR: many few options " << endl;
		cout << "use " << argv[0] << " [options]" << endl;
		cout << "Mandatory :" << endl;
		cout << "  -il   <string>	Learning strings file" << endl;
		cout << "  -it   <string>	Testing strings file" << endl;
		cout << "  -e    <string>	Edit distance costs file" << endl;
		cout << "  -i    <string>	Transducer model file" << endl;
		cout << "Optional :" << endl;
		cout << "  -v    <int> 		Verbosity level" << endl;
		cout << "  -m	 {c,j}		Type of transducer model (default c:conditional)" << endl;
		cout << "  -h 			Print brief help" << endl;
		cout << "  -H 			Print help" << endl;
		exit(-1);
	}

	// read the input file and to build the db file.
	db_ls = input_database(file_ls, verbosity);
	if(verbosity > 3) {
		cout << "Learning strings size: " << db_ls.size() << endl;
	}

	db_ts = input_database(file_ts, verbosity);
	if(verbosity > 3) {
		cout << "Testing strings size: " << db_ts.size() << endl;
	}

	trans_tm = input_transducer(file_tm, verbosity);
	if(verbosity > 2){
		cout << trans_tm << endl;
	}


	alg.initialize(2, db_ls.size(), db_ts.size());
	// Charge the ED costs and the LS set.
	ed = input_edit_distance(file_ed, verbosity);
	alg.set_name(0,"Classic Edit-Distance");

	// To calculate the (x', NN(x')) -- Classic Edit-Distance
	MyString class_x,class_y;
	double de = 0.0;
	double distances[db_ts.size()];

	for(int x = 0; x < db_ts.size(); x++)
		distances[x] = 100000.0;

	for(int x = 0; x < db_ts.size(); x++){
		class_x = db_ts.in(x);
		for(int y = 0; y < db_ls.size(); y++) {
			if(x != y) {
				de = ed(db_ts.out(x), db_ls.out(y));
				if(de < distances[x]){
					class_y = db_ls.in(y);
					distances[x] = de;
				}
			}
		}
		if(class_x(0) == class_y(0))
			alg.set_tp(0, alg.get_tp(0)+1);
	}

	// Conditional and Joint Transducer
	if(trans_model)
		alg.set_name(1, "Joint Transducer Model");
	else
		alg.set_name(1, "Conditional Transducer Model");
	MyString class_tm;

	for(int x = 0; x < db_ts.size();x++)
		distances[x] = 100000.0;

	for(int x = 0; x < db_ts.size(); x++){
		class_x = db_ts.in(x);
		double prob_tm = 0.0;
		class_tm=" ";
		for(int y = 0; y < db_ls.size(); y++){
			if(x != y) {
				// To calculate the conditional probability
				double maxp_tm = max(prob_tm, alg.prob(trans_tm, db_ts.out(x), db_ls.out(y)));
				if(maxp_tm > prob_tm){
					prob_tm = maxp_tm;
					class_tm = db_ls.in(y);
				}
			}
		}
		if((class_x(0) == class_tm(0))==0) {
			alg.set_tp(1, alg.get_tp(1)+1);
		}
	}

	alg.print_result(alg);
	return 0;
}
