#include <iostream>
#include <fstream>
#include "../src/transducer.h"
#include "../src/pair_database.h"
#include "../src/em.h"
 
using namespace std;

//================= Main =========================================

int main (int argc, char *argv[]) {
	int i, argcount=0;
	string file_sed;
	string file_db;
	Pair_database db;
	double precision = 1E-50;
	int verbosity = 0;
	void (*normalize)( Transducer&);

	normalize = normalize_maximum_likelihood;

	//To analize the input arguments
	for (i = 1; i < argc; i++) {

		if (strcmp("-o", argv[i]) == 0) {
			argcount++;
			file_sed = argv[++i];
		} else if (strcmp("-i", argv[i]) == 0) {
			argcount++;
			file_db = argv[++i];
		} else if (strcmp ("-v", argv[i]) == 0) {
			verbosity = atoi(argv[++i]);
		} else if (strcmp("-prec", argv[i]) == 0) {
			precision = atof(argv[++i]);
		} else if (strcmp("-na", argv[i]) == 0) {
			i++;
			if( strcmp("id", argv[i]) == 0 ) {
				normalize = normalize_idempotent;
			} else if( strcmp("c", argv[i]) == 0 ) {
				normalize = normalize_maximum_likelihood;
			} else if( strcmp("j", argv[i]) == 0 ) {
				normalize = normalize_joint;
			} else {
				cout << "ERROR: "
				     << "unknown normalization algorithm "
				     << argv[i] 
				     << "'" << endl;
				exit(-1);
			}
		} else if (strcmp ("-H", argv[i]) == 0) {
			cout << "NAME" << endl;
			cout << "	lsed :  Make up a stochastic transductor, using conditional" <<
				" or joint normalization, from" << endl;
			cout << "		a file with pairs of representation strings." << endl;
			cout << "SYNOPSIS" << endl;
			cout << "	lsed -i {input-file} -o {transducer-file}" << endl;
			cout << "DESCRIPTION" << endl;
			cout << "	From pairs of strings this module obtain the stochastic tranducer" <<
				" model with the use " << endl;
			cout << "	of some normalization model (conditional or joint)." << endl;
			cout << "	Mandatory arguments:" << endl;
			cout << "	-i	Input file with the representation strings of some entity" << endl;
			cout << "	-o	Output file where the stochastic transducer model will be saved" << endl;
			cout << "	Optional arguments" << endl;
			cout << "	-na {c,j}	Normalization mode where c is conditional and j is joint" << endl;
			cout << "			(default c)" << endl;
			cout << "	-prec 	Precision level with will be evaluated the iteration process into" <<
				" EM Algorithm (default 1E-50)" << endl;
			cout << "	-v verbosity level" << endl;
			cout << "	-h brief help" << endl;
			cout << "	-H help" << endl;
			cout << "FILE FORMAT" << endl;
			cout << "	Input File:" << endl;
			cout << endl;
			cout << "	VocabularySize	StringLength(default 0)" << endl;
			cout << "	RepresentationString	RepresentationString" << endl;
			cout << endl;
			cout << "	Output File:" << endl;
			cout << endl;
			cout << "	VocabularySize	StringLength(default 0)" << endl;
			cout << "	TransductionMatrix" << endl;
			cout << endl;
			cout << "SEE ALSO" << endl;
			cout << "	______________" << endl;
			exit(0);
		} else if (strcmp ("-h", argv[i]) == 0) {
			cout << "Mandatory :" << endl;
			cout << "use " << argv[0] << " [options]" << endl;
			cout << "  -i	 <string>	Input file" << endl;
			cout << "  -o	 <string>	Transducer model file" << endl;
			cout << "Optional :" << endl;
			cout << "  -prec <double> 	Precision (default 1E-50)" << endl;
			cout << "  -na   {c,j}		Normalization algorithm (default c)" << endl;
			cout << "  -v    <int> 		Verbosity level" << endl;
			cout << "  -h 			Print brief help" << endl;
			cout << "  -H 			Print help" << endl;
			exit(0);
		} else {
			cout << "ERROR: unknown option " << argv[i] << endl;
			exit(-1);
		}
	}

	if (argcount < 2) {
		cout << "ERROR: many few options " << endl;
		cout << "use " << argv[0] << " [options]" << endl;
		cout << "Mandatory :" << endl;
		cout << "  -i	 <string>	Input file" << endl;
		cout << "  -o	 <string>	Transducer model file" << endl;
		cout << "Optional :" << endl;
		cout << "  -prec <double> 	Precision (default 1E-50)" << endl;
		cout << "  -na   {c,j}		Normalization algorithm (default c)" << endl;
		cout << "  -v    <int> 		Verbosity level" << endl;
		cout << "  -h 			Print brief help" << endl;
		cout << "  -H 			Print help" << endl;
		exit(-1);
	}

	// reading the strings databases generated by the first module (generate_strings).
	if( !file_db.empty() ) {
		ifstream ds(file_db.c_str());
		if( !ds ) {
			cout << "ERROR: I can't open '" 
                             << file_db.c_str() << "'" << endl;
			exit(-1);
		}
		ds >> db;
	}

	if( verbosity > 0 )  {
		cout << "Strings base:" << endl;
  		cout << db;
	}

	Transducer t;
	EM em;
	em.verbosity() = verbosity;
	em.precision() = precision;
	t = em(db, normalize);

	if( verbosity > 0)
        	cout << "iterations: " << em.iterations() << endl;

	if( verbosity > 1 ) {
		cout << "Result:" << endl;
		cout << t << endl;
	}

	if( !file_sed.empty() ) {
		ofstream fsed;
		fsed.open(file_sed.c_str());
		fsed << t;
		fsed.close();
	}
	return 0;
}
