#include <iostream>
#include <fstream>
#include "m-nn.h"

using namespace std;

//================= Main =========================================

int main (int argc, char *argv[]) {
	int i, argcount=0;
	Pair_database db_i, db_o;
	Edit_distance ed;
	string file_o;  
	string file_i;
	string file_ed; 
	int verbosity = 0;

	//To analize the input arguments
	for (i = 1; i < argc; i++) {
		if (strcmp("-i", argv[i]) == 0) {
			argcount++;
			file_i = argv[++i];
		} else if (strcmp("-ed", argv[i]) == 0) {
			argcount++;
			file_ed = argv[++i];
		} else if (strcmp("-o", argv[i]) == 0) {
			argcount++;
			file_o = argv[++i];
		} else if (strcmp ("-v", argv[i]) == 0) {
			verbosity = atoi(argv[++i]);
		} else if (strcmp ("-H", argv[i]) == 0) {
			cout << "NAME" << endl;
			cout << "	m-nn :  By each component in input file, from the same class " <<
				"and using the classical edit distance between " << endl;
			cout << "		strings, obtain the nearest neighbor and save them into output file. " 
				<< endl;
			cout << "SYNOPSIS " << endl;
			cout << "	m-nn -i {input-file} -ed {edit-distance-costs} -o {output-file} " << endl;
			cout << "DESCRIPTION " << endl;
			cout << "	Obtain the nearest neighbor to each component in input file and save" << 
				" them into output file. " << endl;
			cout << "	Mandatory arguments: " << endl;
			cout << "	-i	Input file with the components from any clases " << endl;
			cout << "	-ed	Input file with the edit distance costs " << endl;
			cout << "	-o	Output file where the strings will be saved " << endl;
			cout << "	Optional arguments " << endl;
			cout << "	-v verbosity level " << endl;
			cout << "	-h brief help " << endl;
			cout << "	-H help " << endl;
			cout << "FILE FORMAT " << endl;
			cout << " " << endl;
			cout << "	Input File: Components by class and its representation string " << endl;
			cout << " " << endl;
			cout << "	VocabularySize	StringLength(default 0)" << endl;
			cout << "	Class	String " << endl;
			cout << " " << endl;
			cout << "	Output File: Components and its nearest neighbor " << endl;
			cout << " " << endl;
			cout << "	VocabularySize	StringLength(default 0)" << endl;
			cout << "	String	NearestNeighbor " << endl;
			cout << " " << endl;
			cout << "SEE ALSO " << endl;
			cout << "	m-snn " << endl;
			exit(0);
		} else if (strcmp ("-h", argv[i]) == 0) {
			cout << "use " << argv[0] << " [options]" << endl;
			cout << "Mandatory :" << endl;
			cout << "  -i    <string>	Input file" << endl;
			cout << "  -ed   <string>	Edit distance costs file" << endl;
			cout << "  -o    <string>	Output file" << endl;
			cout << "Optional :" << endl;
			cout << "  -v    <int> 		Verbosity level" << endl;
			cout << "  -h 			Print brief help" << endl;
			cout << "  -H			Print help" << endl;
			exit(0);
		} else {
			cout << "ERROR: unknown option " << argv[i] << endl;
			exit(-1);
		}
	}

	if (argcount < 3) {
		cout << "ERROR: many few options " << endl;
		cout << "use " << argv[0] << " [options]" << endl;
		cout << "Mandatory :" << endl;
		cout << "  -i    <string>	Input file" << endl;
		cout << "  -ed   <string>	Edit distance costs file" << endl;
		cout << "  -o    <string>	Output file" << endl;
		cout << "Optional :" << endl;
		cout << "  -v    <int> 		Verbosity level" << endl;
		cout << "  -h 			Print brief help" << endl;
		cout << "  -H			Print help" << endl;
		exit(-1);
	}

	// read the input file and to build the db file.
	db_i = input_database(file_i, verbosity);
	if(verbosity > 3)
	{
		cout << "Input file size: " << db_i.size() << endl;
	}

	// Charge the ED costs and the LS set.
	ed = input_edit_distance(file_ed, verbosity);

	// To calculate the (x', NN(x')) set
	MyString class_a,class_b;
	double de = 0.0;
	double distances[db_i.size()][db_i.size()];

	for(int i = 0; i < db_i.size(); i++)
		for(int j = 0; j < db_i.size(); j++){
			distances[i][j] = 100000.0;
	}

	for(int i = 0; i < db_i.size(); i++){
		class_a = db_i.in(i);
		for(int j = (i+1); j < db_i.size(); j++) {
			class_b = db_i.in(j);
			if(class_a(0) == class_b(0)) {
				de = ed(db_i.out(i), db_i.out(j));
				distances[i][j] = de;
				distances[j][i] = de;
			}
		}
	}

	for(int i = 0; i < db_i.size(); i++){
		de = distances[i][0];
		int min_j = 0;
		for(int j = 1; j < db_i.size(); j++){
			if( distances[i][j] < de ){
				de = distances[i][j];
				min_j = j;
			}
		}
		db_o.set_voc_size(db_i.get_voc_size());
		db_o.set_str_len(db_i.get_str_len());
		db_o.push_back(db_i.out(i), db_i.out(min_j));
	}

	if( verbosity > 1 )  {
		cout << "Output base:" << endl;
		cout << db_o;
	}
	// to build the pairs of strings file.
	print_database(file_o, db_o);

	return 0;
}
