#include <iostream>
#include <fstream>
#include "../src/transducer.h"
#include "../src/pair_database.h"
#include "../src/trellis.h"
 
using namespace std;

//================= Main =========================================

int main (int argc, char *argv[]) {
	int i, argcount=0;
	string file_transducer;
	Pair_database db;
	int verbosity = 0;
	int voc_size = 0;
	void (*normalize)(Transducer&);

	normalize = normalize_maximum_likelihood;

	//To analize the input arguments
	for (i = 1; i < argc; i++) {
		if (strcmp ("-s", argv[i]) == 0) {
			srand48(atoi(argv[++i]));
		} else if (strcmp("-o", argv[i]) == 0) {
			file_transducer = argv[++i];
			argcount++;
		} else if (strcmp("-vs", argv[i]) == 0) {
			voc_size = atoi( argv[++i]);
			argcount++;
		} else if (strcmp ("-v", argv[i]) == 0) {
			verbosity = atoi(argv[++i]);
		} else if (strcmp ("-na", argv[i]) == 0){
			i++;
			if (strcmp ("j", argv[i]) == 0) {
				normalize = normalize_joint;
			} else if (strcmp ("c", argv[i]) == 0){
				normalize = normalize_maximum_likelihood;
			} else {
				cout << "ERROR: "
				     << "unknown normalization mode <"
				     << argv[i] << ">" << endl;
				exit(-1);
			}
		} else if (strcmp ("-H", argv[i]) == 0) {
			cout << "NAME" << endl;
			cout << "	m-rtrans : Build a random stochastic transducer over some vocabulary." << endl;
			cout << "SYNOPSIS" << endl;
			cout << "	m-rtrans -vs {vocabulary-size} -o {output-file}" << endl;
			cout << "DESCRIPTION" << endl;
			cout << "	This module make up a random transducer over some synthetic vocabulary" <<
				" with a specific size" << endl;
			cout << "	Mandatory arguments:" << endl;
			cout << "	-vs 	Size of the synthetic vocabulary" << endl;
			cout << "	-o 	Name of the output file where the transducer model will be saved" << endl;
			cout << "	Optional arguments" << endl;
			cout << "	-s	Seed to make the transducer" << endl;
			cout << "	-v	Verbosity level" << endl;
			cout << "	-na {c,j}	Normalization mode where c is conditional and j is joint" << endl;
			cout << "			(default c)" << endl;
			cout << "	-h	Brief help" << endl;
			cout << "	-H	Help" << endl;
			cout << "FILE FORMAT" << endl;
			cout << "	Ouput File:" << endl;
			cout << endl;
			cout << "	VocabularySize" << endl;
			cout << "	Transduction Matrix" << endl;
			cout << "SEE ALSO" << endl;
			cout << "	________________" << endl;
			exit(0);
		} else if (strcmp ("-h", argv[i]) == 0) {
			cout << "use " << argv[0] << " [options]" << endl;
			cout << "Mandatory :" << endl;
			cout << "  -vs   <int>		Vocabulary size" << endl;
			cout << "  -o    <string>   	Transducer model file" << endl;
			cout << "Optional :" << endl;
			cout << "  -s    <int> 		Seed" << endl;
			cout << "  -v    <int> 		Verbosity level" << endl;
			cout << "  -na   {c,j}		Normalization algorithm (default c)" << endl;
			cout << "  -h				Print brief help" << endl;
			cout << "  -H				Print help" << endl;
			exit(0);
		} else {
			cout << "ERROR: unknown option " << argv[i] << endl;
			exit(-1);
		}
	}

	if (argcount < 2 ) {
		cout << "ERROR: many few options " << endl;
		cout << "use " << argv[0] << " [options]" << endl;
		cout << "Mandatory :" << endl;
		cout << "  -vs   <int>		Vocabulary size" << endl;
		cout << "  -o    <string>   	Transducer model file" << endl;
		cout << "Optional :" << endl;
		cout << "  -s    <int> 		Seed" << endl;
		cout << "  -v    <int> 		Verbosity level" << endl;
		cout << "  -na   {c,j}		Normalization algorithm (default c)" << endl;
		cout << "  -h   			Print brief help" << endl;
		cout << "  -H   			Print help" << endl;
		exit(-1);
	}

	/*	Creating the transducer with verbosity and precision defined before 
		and applying the EM algorithm. 
	*/

	MyString::initialize(voc_size);
	Transducer target(MyString::voc_size());
	randomize(target);
	if( verbosity > 0 ) {
		cout << "Unnormalized target:" << endl;
		cout << target << endl;
	}

	
	normalize(target);
	if( verbosity > 0 ) {
		cout << "Target:" << endl;
		cout << target << endl;
		check_normalization(target);
	}

	//	Writing the transducer to a file (file_transducer).
	ofstream dt(file_transducer.c_str());
	if( !dt ) {
		cout << "ERROR: I can't open '" 
                            << file_transducer.c_str() << "'" << endl;
		exit(-1);
	}
	dt << target;

	return 0;
}
