#include <iostream>
#include <fstream>
#include "m-sets.h"

using namespace std;

//================= Main =========================================

int main (int argc, char *argv[]) {
	int i, argcount=0;
	int vocabularySize = 0;
	Pair_database db_if, db_fg, db_sg;
	string file_if;  
	string file_fg="";  
	string file_sg="";  
	int NoFG = 0, NoSG=0;
	int seed=0;
	int verbosity = 0;

	//To analize the input arguments
	for (i = 1; i < argc; i++) {
		if (strcmp("-i", argv[i]) == 0) {
			argcount++;
			file_if = argv[++i];
		} else if (strcmp("-fg", argv[i]) == 0) {
			argcount++;
			file_fg = argv[++i];
		} else if (strcmp("-sg", argv[i]) == 0) {
			file_sg = argv[++i];
		} else if (strcmp("-sfg", argv[i]) == 0) {
			argcount++;
			NoFG = atoi(argv[++i]);
		} else if (strcmp("-ssg", argv[i]) == 0) {
			NoSG = atoi(argv[++i]);
		} else if (strcmp ("-s", argv[i]) == 0) {
			seed=atoi(argv[++i]);
		} else if (strcmp ("-v", argv[i]) == 0) {
			verbosity = atoi(argv[++i]);
		} else if (strcmp ("-vs", argv[i]) == 0){
			vocabularySize = atoi(argv[++i]);
		} else if (strcmp ("-H", argv[i]) == 0) {
			cout << "NAME" << endl;
			cout << "	m-sets : Get sfg components from input file and save it into first group file" << 					"and, optionaly, " << endl;
			cout << "		 get all or ssg components that aren't in first group (default) from" <<
				" input file save it" << endl;
			cout << "		 into second group file." << endl;
			cout << "SYNOPSIS" << endl;
			cout << "	m-sets -i {input-file} -fg {first-group} -sfg {size-of-fg}" << endl;
			cout << "	m-sets -i {input-file} -fg {first-group} -sfg {size-of-fg} -sg {second-group}" 
				<< endl;
			cout << "	m-sets -i {input-file} -fg {first-group} -sfg {size-of-fg} -sg {second-group}" <<
				" -ssg {size-of-sg}" << endl;
			cout << "DESCRIPTION" << endl;
			cout << "	Get some components from input file and save it in one or two output files."
				<< endl;
			cout << "	Mandatory arguments:" << endl;
			cout << "	-i 	Input file with the components" << endl;
			cout << "	-fg 	Name of the output file where the first group of components will" << 
				" be saved" << endl;
			cout << "	-sfg	Number of the components that will be saved into fg file" << endl;
			cout << "	Optional arguments" << endl;
			cout << "	-sg 	Name of the output file where the second group of components will" << 
				" be saved" << endl;
			cout << "	-ssg	Number of the components that will be saved into sg file" << endl;
			cout << "	-vs	Vocabulary size (used only if the input file not define it)" << endl;
			cout << "	-s seed to make the strings" << endl;
			cout << "	-v verbosity level" << endl;
			cout << "	-h brief help" << endl;
			cout << "	-H help" << endl;
			cout << "FILE FORMAT" << endl;
			cout << "	Input File:" << endl;
			cout << "" << endl;
			cout << "	OneComponentByLine" << endl;
			cout << "" << endl;
			cout << "	Ouput File:" << endl;
			cout << "" << endl;
			cout << "	OneComponentByLine" << endl;
			cout << "" << endl;
			cout << "SEE ALSO" << endl;
			cout << "	_________________" << endl;
			exit(0);
		} else if (strcmp ("-h", argv[i]) == 0) {
			cout << "use " << argv[0] << " [options]" << endl;
			cout << "Mandatory :" << endl;
			cout << "  -i    <string>	Input file" << endl;
			cout << "  -fg   <string>	First group file" << endl;
			cout << "  -sfg  <int>		Number of components in first group" << endl;
			cout << "Optional :" << endl;
			cout << "  -sg	 <string> 	Second group file" << endl;
			cout << "  -ssg  <int>		Number of components in second group" << endl;
			cout << "			(default components that aren't in first group)" << endl;
			cout << "  -vs			Vocabulary size" << endl;
			cout << "			(used only if the input file not define it)" << endl;
			cout << "  -s    <int>		Seed" << endl;
			cout << "  -v    <int> 		Verbosity level" << endl;
			cout << "  -h 			Print brief help" << endl;
			cout << "  -H 			Print help" << endl;
			exit(0);
		} else {
			cout << "ERROR: unknown option " << argv[i] << endl;
			exit(-1);
		}
	}

	if (argcount < 3) {
		cout << "ERROR: many few options " << endl;
		cout << "use " << argv[0] << " [options]" << endl;
		cout << "Mandatory :" << endl;
		cout << "  -i    <string>	Input file" << endl;
		cout << "  -fg   <string>	First group file" << endl;
		cout << "  -sfg  <int>		Number of components in first group" << endl;
		cout << "Optional :" << endl;
		cout << "  -sg	 <string>	Second group file" << endl;
		cout << "  -ssg  <int>		Number of components in second group" << endl;
		cout << "			(default components that aren't in first group)" << endl;
		cout << "  -vs			Vocabulary size" << endl;
		cout << "			(used only if the input file not define it)" << endl;
		cout << "  -s    <int>		Seed" << endl;
		cout << "  -v    <int> 		Verbosity level" << endl;
		cout << "  -h 			Print brief help" << endl;
		cout << "  -H 			Print help" << endl;
		exit(-1);
	}

	// read the input file and to build the db file.
	if(vocabularySize > 0){
		db_if = input_database_vocabulary(file_if, verbosity);
		db_if.set_voc_size(vocabularySize);
	} else {
		db_if = input_database(file_if, verbosity);
	}
	if(verbosity > 1)
	{
		cout << "Input file size : " << db_if.size() << endl;
	}
	// to build the test and learning strings files.
	if(verbosity > 1)
		cout << "Number of FG : " << NoFG << endl;
	if(NoSG == 0) { NoSG = db_if.size() - NoFG - 1; }
	if(verbosity > 1)
		cout << "Number of SG : " << NoSG << endl;

	if(NoFG > 0 && !file_fg.empty()){
		db_fg = set_string(db_if, file_fg, NoFG, seed);
		db_fg.set_voc_size(db_if.get_voc_size());
		db_fg.set_str_len(0);
		if( verbosity > 3 )  {
			cout << "First group base:" << endl;
			cout << db_fg;
		}
		print_database(file_fg, db_fg);
	}else {
		if(file_fg.empty() && NoFG > 0)
			cout << "Error: FG file is not defined." << endl;
		if(!file_fg.empty() && (NoFG == 0))
			cout << "Error: FG size is not defined." << endl;
	}

	if(!file_sg.empty()){
		db_sg = set_string(db_if, file_sg, NoSG, seed);
		db_sg.set_voc_size(db_if.get_voc_size());
		db_sg.set_str_len(0);
		if( verbosity > 3 )  {
			cout << "Second group base:" << endl;
			cout << db_sg;
		}
		print_database(file_sg, db_sg);
	}

	return 0;
}
