#include <iostream>
#include <fstream>
#include "../src/transducer.h"
#include "../src/pair_database.h"
 
using namespace std;

//================= Main =========================================

int main (int argc, char *argv[]) {
	int i, argcount=0;
	string file_string;
	string file_transducer;
	string file_db;
	Pair_database db;
	int verbosity = 0;

	//To analize the input arguments
	for (i = 1; i < argc; i++) {

		if (strcmp ("-i", argv[i]) == 0) {
			argcount++;
			file_string = argv[++i];
		} else if (strcmp("-t", argv[i]) == 0) {
			argcount++;
			file_transducer = argv[++i];
		} else if (strcmp("-o", argv[i]) == 0) {
			argcount++;
			file_db = argv[++i];
		} else if (strcmp ("-v", argv[i]) == 0) {
			verbosity = atoi(argv[++i]);
		} else if (strcmp ("-H", argv[i]) == 0) {
			cout << "NAME" << endl;
			cout << "	m-snn : By each string in the input file, using a stochastic" <<
				" transducer model, this module" << endl;
			cout << "		generate a output string and save them into output file." << endl;
			cout << "SYNOPSIS" << endl;
			cout << "	m-snn -i {input-file} -t {transducer-model} -o {output-file}" << endl;
			cout << "DESCRIPTION" << endl;
			cout << "	From transduction process obtain a output string from each string" << 
				" into the input file." << endl;
			cout << "	Mandatory arguments:" << endl;
			cout << "	-i	Input file with the strings" << endl;
			cout << "	-t	Input file with the stochastic transducer model" << endl;
			cout << "	-o	Output file where the strings will be saved" << endl;
			cout << "	Optional arguments" << endl;
			cout << "	-v verbosity level" << endl;
			cout << "	-h brief help" << endl;
			cout << "	-H help" << endl;
			cout << endl;
			cout << "FILE FORMAT" << endl;
			cout << endl;
			cout << "	Input File: " << endl;
			cout << endl;
			cout << "	VocabularySize	StringLength(default 0)" << endl;
			cout << "	OneStringByLine" << endl;
			cout << "	..." << endl;
			cout << endl;
			cout << "	Output File: " << endl;
			cout << endl;
			cout << "	VocabularySize	StringLength(default 0)" << endl;
			cout << "	InputString	OutputString" << endl;
			cout << "	...		..." << endl;
			cout << endl; 
			cout << "SEE ALSO" << endl;
			cout << "	m-nn" << endl;
			exit(0);
		} else if (strcmp ("-h", argv[i]) == 0) {
			cout << "use " << argv[0] << " [options]" << endl;
			cout << "Mandatory :" << endl;
			cout << "  -i	<string>	Input String file" << endl;
			cout << "  -t	<string>	Transducer model file" << endl;
			cout << "  -o	<string>	Output String file" << endl;
			cout << "Optional :" << endl;
			cout << "  -v   <int> 		Verbosity level" << endl;
			cout << "  -h 			Print brief help" << endl;
			cout << "  -H 			Print help" << endl;
			exit(0);
		} else {
			cout << "ERROR: unknown option " << argv[i] << endl;
			exit(-1);
		}
	}

	if (argcount < 3) {
		cout << "ERROR: many few options " << endl;
		cout << "use " << argv[0] << " [options]" << endl;
		cout << "Mandatory :" << endl;
		cout << "  -i	<string>	Input String file" << endl;
		cout << "  -t	<string>	Transducer model file" << endl;
		cout << "  -o	<string>	Output String file" << endl;
		cout << "Optional :" << endl;
		cout << "  -v   <int> 		Verbosity level" << endl;
		cout << "  -h 			Print brief help" << endl;
		cout << "  -H 			Print help" << endl;
		exit(-1);
	}

	// reading the strings databases generated by the first module (generate_strings).
	if( !file_string.empty() ) {
		ifstream ds(file_string.c_str());
		if( !ds ) {
			cout << "ERROR: I can't open '" 
                             << file_string.c_str() << "'" << endl;
			exit(-1);
		}
		ds > db;
	}
	else{
		exit(-1);
	}

	if( verbosity > 1 )  {
		cout << "Strings base:" << endl;
		cout << db;
	}

	// reading the transducer from a file created by other module (m-rtrans).
	Transducer t;
	if( !file_transducer.empty() ) {
		ifstream dt(file_transducer.c_str());
		if( !dt ) {
			cout << "ERROR: I can't open '" 
       	                     << file_transducer.c_str() << "'" << endl;
			exit(-1);
		}
		dt >> t;
	}
	else{
		exit(-1);
	}	

	if( verbosity > 0 ) {
		cout << "Result:" << endl;
		cout << t << endl;
	}

	MyString::initialize(t.size());
	Pair_database db2 = random_new_database(t, db.size(), db.get_str_len(), db);
	if( verbosity > 3 )  {
		cout << "Database:" << endl;
  		cout << db2;
	}

	if( !file_db.empty() ) {
		ofstream fdb;
		fdb.open(file_db.c_str());
		fdb << db2;
		fdb.close();
	}
	return 0;
}
