#include <vector>
#include "../src/pair_database.h"
#include "./src/automaton.h"
 
using namespace std;


//================= Main =========================================

int main (int argc, char *argv[]) {
	int i, argcount=0;
	string file_strings;
	string file_prob;
	Pair_database db;
	int db_size = 0;
	int verbosity = 0;
	Automata automata;

	//To analize the input arguments
	for (i = 1; i < argc; i++) {

		if (strcmp ("-o", argv[i]) == 0) {
			file_strings = argv[++i];
			argcount++;
		} else if (strcmp("-dbs", argv[i]) == 0) {
			db_size = atoi(argv[++i]);
			argcount++;
		} else if (strcmp("-i", argv[i]) == 0) {
			file_prob = argv[++i];
			argcount++;
		} else if (strcmp ("-v", argv[i]) == 0) {
			verbosity = atoi(argv[++i]);
		} else if (strcmp ("-s", argv[i]) == 0) {
			srand48(atoi(argv[++i]));
		} else if (strcmp ("-H", argv[i]) == 0) {
		cout << "NAME" << endl;
		cout << "	m-strings : Make strings using a one-state automata	" << endl;
		cout << "SYNOPSIS" << endl;
		cout << "	m-string [OPTIONS] -i {input-file} -o {output-file} -dbs {output-size}" << endl;
		cout << "DESCRIPTION" << endl;
		cout << "	Make output-size strings using a stochastic automata with input-file probabilities" << endl;
		cout << "	and save it into output-file.	" << endl;
		cout << "Mandatory arguments:" << endl;
		cout << "	-i input file with transduction probabilities" << endl;
		cout << "	-o output file to save the output strings" << endl;
		cout << "	-dbs number of generated strings" << endl;
		cout << "Optional arguments" << endl;
		cout << "	-s seed to make the strings" << endl;
		cout << "	-v verbosity level" << endl;
		cout << "	-h brief help" << endl;
		cout << "	-H help" << endl;
		cout << "FILE FORMAT" << endl;
		cout << "	Input file: The end symbol (#) belong to symbols defined in this file" << endl;
		cout << "	Symbol	ProbabilityOfSymbol" << endl << endl;
		cout << "	Output file: The string length is 0 because the strings are generated" <<
			" using probabilities model" << endl;
		cout << "	VocabularySize	StringLength" << endl;
		cout << "	OneStringByLine" << endl << endl;
		cout << "SEE ALSO" << endl;
		cout << "	strings" << endl;
		exit(0);
		} else if (strcmp ("-h", argv[i]) == 0) {
			cout << "use " << argv[0] << " [options]" << endl;
			cout << "Mandatory :" << endl;
			cout << "  -i   <string>   	Probabilities file" << endl;
			cout << "  -o   <string>   	String file" << endl;
			cout << "  -dbs <string>      	Database size" << endl;
			cout << "Optional :" << endl;
			cout << "  -s   <int> 		seed" << endl;
			cout << "  -v   <int> 		verbosity level" << endl;
			cout << "  -h			print brief help" << endl;
			cout << "  -H			print help" << endl;

			exit(0);
		} else {
			cout << "ERROR: unknown option " << argv[i] << endl;
			exit(-1);
		}
	}
	if (argcount < 3) {
		cout << "ERROR: many few options " << endl;
		cout << "use " << argv[0] << " [options]" << endl;
		cout << "Mandatory :" << endl;
		cout << "  -i   <string>   	Probabilities file" << endl;
		cout << "  -o   <string>   	String file" << endl;
		cout << "  -dbs <string>      	Database size" << endl;
		cout << "Optional :" << endl;
		cout << "  -s   <int> 		seed" << endl;
		cout << "  -v   <int> 		verbosity level" << endl;
		cout << "  -h			print brief help" << endl;
		cout << "  -H			print help" << endl;
		exit(-1);
	}

	if( !file_prob.empty() ) {
		ifstream dt(file_prob.c_str());
		if( !dt ) {
			cout << "ERROR: I can't open '" 
       	                     << file_prob.c_str() << "'" << endl;
			exit(-1);
		}
		dt >> automata;
	}
	
	write_strings(automata, db_size, file_strings);

	return 0;
}
