#include <vector>
#include "matrix.h"
#include "mystring.h"
#include "edit_distance.h"

double min( double a, double b, double c ) {
        if( a < b && a < c )
                return a;
        if( b < c )
                return b;
        return c;
}



void Edit_distance::initialize( int voc_size ) {

	f_iw.resize(voc_size + 1);
	f_dw.resize(voc_size + 1);
	f_sw.initialize(voc_size + 1, voc_size + 1);
}


double Edit_distance::operator()( 
		const MyString &s1, 
		const MyString &s2 ) const {

        double mat[s1.size()+1][s2.size()+1];

        for( int i = 0; i <= s1.size(); i++ )
                for( int j = 0; j <= s2.size(); j++ ) {
                        if( i == 0 && j == 0 )
                                mat[i][j] = 0;
                        else if( i > 0 && j == 0 )
                                mat[i][0] = mat[i-1][0] + f_iw[s1(i-1)];
                        else if( i == 0 && j > 0 )
                                mat[0][j] = mat[0][j-1] + f_dw[s2(j-1)];
                        else if(s1(i-1) == s2(j-1) )
                                mat[i][j] = min( mat[i-1][j] + f_iw[s1(i-1)],
                                                 mat[i][j-1] + f_dw[s2(j-1)],
                                                 mat[i-1][j-1] );
                        else
                                mat[i][j] = min( mat[i-1][j] + f_iw[s1(i-1)],
                                                 mat[i][j-1] + f_dw[s2(j-1)],
                                                 mat[i-1][j-1] 
						     + f_sw(s1(i-1), s2(j-1)));
                }
        return mat[s1.size()][s2.size()];
}

ostream& operator<<( ostream& os, Edit_distance &ed) {

	os << ed.size() << endl;

	for( int i = 0; i <= ed.size(); i++ ) {
		os << ed.ins(i) << " ";
	}
	os << endl;

	for (int i = 0; i < ed.size(); i++){
		os << ed.del(i) << " ";
		for (int j = 0; j < ed.size(); j++) {
			os << ed.sub(i,j) << " ";
		}
		os << endl;
	}

	return os;

}

istream& operator>>( istream& is, Edit_distance &ed) {
	int size;

	is >> size;
	ed.initialize(size);

	for( int i = 0; i <= size; i++ ) {
		is >> ed.ins(i);
	}

	for (int i = 0; i <= size; i++){
		is >> ed.del(i);
		for (int j = 0; j < size; j++) {
			is >> ed.sub(i,j);
		}
	}

	return is;

}



