#include <values.h>
#include "math.h"
#include "em.h"

using namespace std;

void expectation(       Transducer &count, 
		  const Transducer &t,
		  const Trellis    &alpha, 
		  const Trellis    &beta,
		  const MyString   &in, 
		  const MyString   &out) {

	for (int i = 0; i <= in.size(); i++)
		for (int j = 0; j <= out.size(); j++) {
			if (i < in.size())
				count.del(in(i)) += alpha(i,j) 
					* t.del(in(i)) * beta(i + 1,j) 
					/ beta(0,0);
			if (j < out.size())
				count.ins(out(j)) += alpha(i,j) 
					* t.ins(out(j)) * beta(i,j+1) 
					/ beta(0,0);
			if (i < in.size() && j < out.size())
				count.sub(in(i),out(j)) += alpha(i,j)
					* t.sub(in(i),out(j)) 
					* beta(i + 1,j + 1) / beta(0,0);
		}
	count.end()++;
}

Transducer EM::operator()( Pair_database &db, 
		void (*normalize)( Transducer&) ) {
        Transducer t(MyString::voc_size());

	double acc_ant = -MAXDOUBLE;
	randomize(t);
	normalize(t);

	if (f_verbosity > 1) {
		cout << "Random transducer:" << endl;
		cout << t << endl;
	}
//        check_normalization(t);

	int num_iter = 0;
	double acc = 0.0;
	while (true) {
		num_iter++;
		Transducer count(MyString::voc_size());
		nullify(count);
		if( f_verbosity > 2 ) {
			cout << "Null transducer:" << endl;
			cout << count << endl;
		}
		acc = 0.0;
		for (int i = 0; i < db.size(); i++) {
			Trellis alpha = forward (t, db.in(i), db.out(i));
			if( f_verbosity > 3 ) {
				cout << "Alpha:" << endl;
				cout << alpha;
			}
			Trellis beta = backward(t, db.in(i), db.out(i));
			if( f_verbosity > 3 ) {
				cout << "Beta:" << endl;
				cout << beta << endl; 
			}
			expectation (count, t, alpha, beta, 
					db.in(i), db.out(i));
			if( f_verbosity > 2 ) {
				cout << "Expectations:" << endl;
				cout << count << endl;
			}
			acc += log (beta(0,0) * t.end());
		}
                if( f_verbosity > 0 ) {
			cout << " iter: " << num_iter
			     << " prob: " << acc 
			     << " rel. error : " << (acc_ant-acc )/acc << endl;
		}
		if (acc <= acc_ant) {
//                if ( (acc_ant - acc) / acc < f_precision ) {
			break;
		}
		acc_ant = acc;
		if (f_verbosity > 1) {
			cout << "----------------------------" << endl;
			cout << "unnormalized count:" << endl;
			cout << count << endl;
		}
		t = count;
		if (f_verbosity > 1) {
			cout << "unnormalized t:" << endl;
			cout << t << endl;
		}
		normalize(t);
		if (f_verbosity > 1) {
			cout << "New transducer:" << endl;
			cout << t << endl;
			cout << "----------------------------" << endl;
		}
	}
	
	iter = num_iter;
	prob = acc;

	return t;

}

double log_prob( 
		const Transducer    &t, 
		const Pair_database &db ) {

	double acc;

	acc = 0.0;
	for( int i = 0; i < db.size(); i++ ) {
		Trellis beta = backward(t, db.in(i), db.out(i));
		acc += log( beta(0,0) * t.end());
	}
	return acc;
}

