#include <iostream>
#include "matrix.h"

void nullify( Matrix &m ) {
        for( int i = 0; i < m.dimX(); i++ )
                for( int j = 0; j < m.dimY(); j++ )
                        m(i,j) = 0.0;
}

void randomize( Matrix &m ) {
        for( int i = 0; i < m.dimX(); i++ )
                for( int j = 0; j < m.dimY(); j++ )
                        m(i,j) = drand48();
}

void uniform2( Matrix &m ) {
        for( int i = 0; i < m.dimX(); i++ )
                for( int j = 0; j < m.dimY(); j++ )
                        m(i,j) = 1.0 / m.dimY();
}

void normalize2( Matrix &m ) {
        for( int i = 0; i < m.dimX(); i++ ) {
                double sum = 0.0;
                for( int j = 0; j < m.dimY(); j++ )
                      sum +=m(i,j);
                for( int j = 0; j < m.dimY(); j++ )
                      m(i,j) = m(i,j) / sum ;
	}
}			
void random_output(
		const Matrix	&m,
		const MyString   &in,
		      MyString   &out ){

	out.clear();
	int i = 0;
	while( true ) {

		double r;
		double acc;
		int v;
		bool more_insertions;
		do {
		        more_insertions = false;
			r = drand48();
			acc = 0.0;

			//-----  insertions -----
			for( v = 0; v < m.size(); v++ ) {
				acc += m.ins(v);
				if( acc > r ) {
					out.push_back(v);
					more_insertions = true;
					break;
				}
			}
		} while( more_insertions);

		if( i == in.size() )
			break;

		//-----  substitutions -----
		for( v = 0; v < m.size(); v++ ) {
			acc += m.sub(in(i),v);
			if( acc > r ) {
				out.push_back(v);
				break;
			}
		}

		//----- deletions -----
		// if acc < r I do nothing (I skip the input symbol)
		acc += m.del(in(i));
		if( acc > 1.0 + 1E-8) {
			cerr << "ERROR in random_output" << endl;
			abort();
		}

		i++;
	}
}


ostream& operator<<( ostream& os, const Matrix &m) {
	for( int i = 0; i < m.dimX(); i++ ) {
		for( int j = 0; j < m.dimY(); j++ )
			os << m(i,j) << " ";
		os << endl;
	}
	return os;
}
