#include <iostream>
#include <vector>
#include <map>
#include "mystring.h"

using namespace std;

class Vocabulary {
	vector<char> sym;
	map<char,int> cod;

	public:

	int size() {
		return sym.size();
	}

	void initialize(int l ) {
		sym.clear();
		cod.clear();
		for( int i = 0; i < l; i++ ) {
			char c = 'a' + i;
			sym.push_back(c);
			cod[c] = i;
		}
	}

	char symbol( unsigned c ) const {
		if( c >= sym.size() ) {
			cerr << "ERROR in symbol: unknown code" << endl;
			abort();
		}
		return sym[c];
	}

	int code( char s ) {
		if( cod.count(s) == 0 ) {
			cod[s] = sym.size();
			sym.push_back(s);
		}
		return cod[s];
	}
			
};

static Vocabulary v;

MyString::MyString(const string &s ) {
	clear();
	for( unsigned i = 0; i < s.size(); i++ ) {
		push_back(v.code(s[i]));
	}
}

void MyString::operator=( const string &s) {
	clear();
	for( unsigned i = 0; i < s.size(); i++ ) {
		push_back(v.code(s[i]));
	}
}

int MyString::voc_size(){
	return v.size();
}

int MyString::code(char c){
	return v.code(c);
}

void MyString::initialize(int i){
	v.initialize(i);
}

ostream& operator<<(ostream &os, MyString s) {
	for( int i = 0; i < s.size(); i++ )
		os << v.symbol(s(i));
	return os;

}

istream& operator>>(istream &is, MyString &s) {
	s.clear();
	string buf;
	is >> buf;
	for( unsigned i = 0; i < buf.size(); i++ ) {
		s.push_back(v.code(buf[i]));
	}
	return is;
}

MyString 
random_MyString(unsigned l) {
	MyString s;
	for( unsigned i = 0; i < l; i++ )
		s.push_back(int(drand48() * v.size()));
	return s;
}

