#ifndef pair_database_h
#define pair_database_h

#include <iostream>
#include <fstream>
#include <vector>
#include "mystring.h"
#include "transducer.h"

struct Pair_database {
	vector<MyString> f_in;
	vector<MyString> f_out;
	vector<bool>	state;  // available
	int voc_size;
	int str_len;

	Pair_database() {
		f_in.clear();
		f_out.clear();
	}

	void clear() {
		f_in.clear();
		f_out.clear();
	}


	int size() const {
		return f_in.size();
	}

	MyString in(int i) const {
		return f_in[i];
	}

	MyString out( int i) const {
		return f_out[i];
	}

	void push_back( const MyString &_in, const MyString &_out) {
		f_in.push_back(_in);
		f_out.push_back(_out);
		state.push_back(true); // available state
	}

	void set_voc_size(int s)
	{
		voc_size = s;
	}

	int get_voc_size()
	{
		return voc_size;
	}

	void set_str_len(int l)
	{
		str_len = l;
	}

	int get_str_len()
	{
		return str_len;
	}

	void set_state(bool s, int index)
	{
		state[index] = s;
	}

	bool get_state(int index) const {
		return state[index];
	}

};

istream& operator>>( 
		istream& is, 
		Pair_database &db );

istream& operator+( 
		istream& is, 
		Pair_database &db);

istream& operator>( istream& is, Pair_database &db);

ostream& operator<<( 
		ostream& os, 
		Pair_database &db);

ostream& operator-( 
		ostream& os, 
		Pair_database &db);

void random_pair_database( 
		Transducer    &t,
                const int           db_size, 
		const int           len,
                      Pair_database &db );

Pair_database random_new_database( 
		Transducer     &t,
		const int            db_size,
		const int            len,
		      Pair_database  &db );

void random_in_database( 
		const int      db_size,
		const int      len,
		Pair_database  &db );



#endif

