#include "transducer.h"
#include "math.h"

#include <iostream>

void nullify( Transducer &t) {
	for( int i = 0; i < t.size(); i++ )
		for( int j = 0; j < t.size(); j++ )
			t.sub(i,j) = 0.0;

	for( int i = 0; i < t.size(); i++ ) {
		t.ins(i) = 0.0;
		t.del(i) = 0.0;
	}
	t.end() = 0.0;
}	

void randomize( Transducer &t) {
	for( int i = 0; i < t.size(); i++ )
		for( int j = 0; j < t.size(); j++ )
			t.sub(i,j) = drand48();

	for( int i = 0; i < t.size(); i++ ) {
		t.ins(i) = drand48();
		t.del(i) = drand48();
	}
	t.end() = drand48();
}
	
void initialize_sw( Transducer &t, double sw ) {
	if( t.size() <= 0 ) {
		cout << "ERROR in Transducer::initialize(): "
			<< "the transducer was not initialized" << endl;
		abort();
	}
	double rw = (1.0 - sw ) / (2 * t.size())  ;

	for( int i = 0; i < t.size(); i++ )
		for( int j = 0; j < t.size(); j++ )
			if( i == j )
				t.sub(i,j) = sw;
			else
				t.sub(i,j) = rw;


	for( int i = 0; i < t.size(); i++ ) {
		t.ins(i) = rw;
		t.del(i) = rw;
	}
	t.end() = sw + t.size() *  rw;
}
	
void normalize_joint( Transducer &t) {
	int i, j;
	double N = 0.0;
	double N_I;
	double N_S;
	double N_D;

  	N_I = 0.0;
  	for (j = 0; j < t.size(); j++)
    		N_I += t.ins(j);

  	N_D = 0.0;
  	for (i = 0; i < t.size(); i++)
    		N_D += t.del(i);

	N_S = 0.0;
	for( i = 0; i < t.size(); i++ )
		for( j = 0; j < t.size(); j++ )
			N_S += t.sub(i,j);

	N = N_I + N_S + N_D + t.end();

// Normalize
	t.end() = 1.0 - N_I / N;
        for( j = 0; j < t.size(); j++ ) {
		t.ins(j) = t.ins(j) / N;
	}

        for( j = 0; j < t.size(); j++ ) {
		t.del(j) = t.del(j) / N;
	}

	for (i = 0; i < t.size(); i++)
	{
		for (j = 0; j < t.size(); j++)
			t.sub(i,j) = t.sub(i,j) / N; 
	}
}


void normalize_maximum_likelihood( Transducer &t) {
	int i, j;
	double N;
	double N_I;
	double N_S;
	double N_D;
	double N_a;

  	N_I = 0.0;
  	for (j = 0; j < t.size(); j++)
    		N_I += t.ins(j);

  	N_D = 0.0;
  	for (i = 0; i < t.size(); i++)
    		N_D += t.del(i);

	N_S = 0.0;
	for( i = 0; i < t.size(); i++ )
		for( j = 0; j < t.size(); j++ )
			N_S += t.sub(i,j);

	N = N_I + N_S + N_D + t.end();

        for( j = 0; j < t.size(); j++ ) {
		t.ins(j) = t.ins(j) / N;
	}

	t.end() = 1.0 - N_I / N;
	
	for (i = 0; i < t.size(); i++)
	{
		N_a = t.del(i);
		for (j = 0; j < t.size(); j++)
			N_a += t.sub(i,j);
		
		t.del(i) = t.del(i) / N_a * t.end(); 
		for (j = 0; j < t.size(); j++)
			t.sub(i,j) = t.sub(i,j) / N_a * t.end(); 
	}
}


void normalize_idempotent( Transducer &t) {
	int i, j;
	double N;
	double N_I; // N(\lambda)
	double N_S;
	double N_D; 

  	N_I = 0.0;
  	for (j = 0; j < t.size(); j++)
    		N_I += t.ins(j);

  	N_D = 0.0;
  	for (i = 0; i < t.size(); i++)
    		N_D += t.del(i);

	N_S = 0.0;
	for( i = 0; i < t.size(); i++ )
		for( j = 0; j < t.size(); j++ )
			N_S += t.sub(i,j);

	N = N_I + N_S + N_D + t.end();
	
	double Lambda = N + t.size() * N_I;

        for( j = 0; j < t.size(); j++ ) {
		t.ins(j) = t.ins(j) / Lambda;
	}

	double factor = (Lambda - N_I) / Lambda;

	t.end() = factor;

	for (i = 0; i < t.size(); i++)
	{
		double N_a = t.del(i);
		for (j = 0; j < t.size(); j++)
			N_a += t.sub(i,j);

		t.del(i) = t.del(i) / N_a *  factor; 
		for (j = 0; j < t.size(); j++)
			t.sub(i,j) = t.sub(i,j) / N_a * factor; 
	}
	
	
	
}

/*
void normalize( Transducer &t) {
	int i, j;
	double N;
	double N_I;
	double N_S;
	double N_D;
	double N_a;

  	N_I = 0.0;
  	for (j = 0; j < t.size(); j++)
    		N_I += t.ins(j);

  	N_D = 0.0;
  	for (i = 0; i < t.size(); i++)
    		N_D += t.del(i);

	N_S = 0.0;
	for( i = 0; i < t.size(); i++ )
		for( j = 0; j < t.size(); j++ )
			N_S += t.sub(i,j);

	N = N_I + N_S + N_D + t.end();

        for( j = 0; j < t.size(); j++ ) {
		t.ins(j) = t.ins(j) / (N_I + t.end());
	}

	for (i = 0; i < t.size(); i++)
	{
		N_a = t.del(i);
		for (j = 0; j < t.size(); j++)
			N_a += t.sub(i,j);
		
		t.del(i) = t.del(i) / N_a * t.end() / ( N_I + t.end()); 
		for (j = 0; j < t.size(); j++)
			t.sub(i,j) = t.sub(i,j) / N_a * 
				t.end() / (N_I + t.end()); 
	}

	t.end() = t.end() / ( N_I + t.end());
}
*/

void check_normalization( Transducer &t)
{
	double w_ins;
	double w_lin;
	int i, j;

	w_ins = 0.0;
	for (j = 0; j < t.size(); j++)
		w_ins += t.ins(j);

	for (i = 0; i < t.size(); i++) {
		w_lin = t.del(i);
		for (j = 0; j < t.size(); j++)
			w_lin += t.sub(i,j);
		if (fabs (w_ins + w_lin - 1.0) >= 1E-8){
			cerr << "ERROR: check_normalization" << endl;
			abort();
		}
	}
	if (fabs (w_ins + t.end() - 1.0) >= 1E-8){
		cerr << "ERROR: check_normalization" << endl;
		abort();
	}
}

double dist( const Transducer &t1, const Transducer &t2 ) {
	double acc = 0.0;

	if( t1.size() != t2.size() ) {
		cerr << "ERROR: I canot compute the distance" << endl;
		abort();
	}
	for( int i = 0; i < t1.size(); i++ )
		acc += pow( t1.ins(i) - t2.ins(i), 2.0 );
	for( int i = 0; i < t1.size(); i++ )
		acc += pow( t1.del(i) - t2.del(i), 2.0 );
	for( int i = 0; i < t1.size(); i++ )
		for( int j = 0; j < t1.size(); j++ )
			acc += pow( t1.sub(i,j) - t2.sub(i,j), 2.0 );
	acc += pow( t1.end() - t2.end(), 2.0 );
	return sqrt(acc);
}

void random_output( 
		Transducer &t, 
		MyString   &in,
		      MyString   &out ){

	out.clear();
	int i = 0;
	while( true ) {

		double r;
		double acc;
		int v;
		bool more_insertions;

		do {
		        more_insertions = false;
			r = drand48();
			acc = 0.0;
	
			//-----  insertions -----
			for( v = 0; v < t.size(); v++ ) {
				acc += t.ins(v);
				if( acc > r ) {
			
					out.push_back(v);
					more_insertions = true;
					break;
				}
			}
			
		} while( more_insertions);

		if( i == in.size() )
			break;

		//-----  substitutions -----
		for( v = 0; v < t.size(); v++ ) {
			acc += t.sub(in(i),v);
			if( acc > r ) {
				out.push_back(v);
				break;
			}
		}

		//----- deletions -----
		// if acc < r I do nothing (I skip the input symbol)

		acc += t.del(in(i));
		/*if( acc > 1.0 + 1E-8) {
			cerr << "ERROR in random_output" << endl;
			abort();
		}*/
		i++;
	}
}

ostream& operator<<( ostream& os, Transducer &t) {
	int i, j;
//	double acc;

	os << t.size() << endl;
	os << t.end() << " ";
	
	for( int i = 0; i < t.size(); i++ ) {
		os << t.ins(i) << " ";
	}
//	os << "= " << acc;
	os << endl;

	for (i = 0; i < t.size(); i++){
//		acc = 0;
		os << t.del(i) << " ";
//		acc += t.del(i);
		for (j = 0; j < t.size(); j++) {
			os << t.sub(i,j) << " ";
//			acc += t.sub(i,j);
		}
//		os << "= " << acc;
		os << endl;
	}

	return os;
}

istream& operator>>( istream& is, Transducer &t) {
	int size;

	is >> size;
	t.initialize(size);

	is >> t.end();
	for( int i = 0; i < t.size(); i++ ) {
		is >> t.ins(i);
	}

	for (int i = 0; i < t.size(); i++){
		is >> t.del(i);
		for (int j = 0; j < t.size(); j++) {
			is >> t.sub(i,j);
		}
	}


	return is;

}


