#ifndef transducer_h
#define transducer_h

#include <iostream>
#include "matrix.h"
#include "mystring.h"

using namespace std;

#include <iostream>
class Transducer {
	Matrix mat;

	public: 

	Transducer( int _size = MyString::voc_size()) : 
		mat( _size + 1, _size + 1){}

	Transducer( const Transducer &t) : mat(t.mat){}

	void operator=( const Transducer &t) {
		mat = t.mat;
	}

	void initialize( int _size) {
		mat.initialize( _size+1, _size+1);
        }

	int size() const {
		return mat.dimX() - 1;
	}

	double& sub( int i, int j ) {
		return mat( i + 1, j + 1 );
	}

	double sub( int i, int j ) const {
		return mat( i + 1, j + 1 );
	}

	double& ins( int j ) {
		return mat(0, j + 1);
	}

	double ins( int j ) const {
		return mat(0, j + 1);
	}

	double& del( int i ) {
		return mat( i + 1, 0 );
	}

	double del( int i ) const {
		return mat( i + 1, 0 );
	}

	double& end() {
		return mat(0,0);
	}

	double end() const {
		return mat(0,0);
	}
};


void nullify( Transducer &t);
void randomize( Transducer &t);
void initialize_sw( Transducer &t, double sw = 0.9);
void normalize_joint( Transducer &t);
void normalize_maximum_likelihood( Transducer &t);
void normalize_idempotent( Transducer &t);
void normalize( Transducer &t);
void check_normalization(Transducer &t);
double dist( const Transducer &t1, const Transducer &t2 );
void random_output(Transducer &t, MyString   &in, MyString &out);
double prob_x_y(const Transducer &t, const MyString   &x, const MyString &y);
ostream& operator<<( ostream& os, Transducer &t);
istream& operator>>( istream& is, Transducer &t);

#endif
