#include <iostream>
#include "trellis.h"
#include "math.h"

Trellis forward (
		const Transducer &t,  
		const MyString   &in, 
		const MyString   &out) {

	Trellis alpha(in.size()+1, out.size()+1);

	for (int i = 0; i <= in.size(); i++)
		for (int j = 0; j <= out.size(); j++) {
			alpha(i,j) = 0.0;
			if (i == 0 && j == 0)
				alpha(i,j) = 1.0;
			if (i > 0)
				alpha(i,j) += alpha(i-1,j) 
					* t.del(in(i - 1));
			if (j > 0)
				alpha(i,j) += alpha(i,j-1) 
					* t.ins(out(j - 1));
			if (i > 0 && j > 0)
				alpha(i,j) += alpha(i-1,j-1)
					* t.sub(in(i - 1),out(j - 1));
		}
	return alpha;
}

Trellis backward (
		const Transducer &t, 
		const MyString   &in, 
		const MyString   &out) {

//	cout << "in " << in.size()+1 << " out " << out.size()+1 << endl;;
	Trellis beta(in.size()+1,out.size()+1);

	for (int i = in.size(); i >= 0; i--)
		for (int j = out.size(); j >= 0; j--) {
			beta(i,j) = 0.0;
			if (i == in.size() && j == out.size())
				beta(i,j) = 1.0;
			if (i < in.size())
				beta(i,j) += beta(i + 1,j) 
					* t.del(in(i));
			if (j < out.size())
				beta(i,j) += beta(i, j + 1) 
					* t.ins(out(j));
			if (i < in.size() && j < out.size())
				beta(i,j) += beta(i + 1, j + 1)
					* t.sub(in(i),out(j));
		}
	return beta;
}

