#include <iostream>
#include <fstream>
#include "../src/pair_database.h"
 
using namespace std;

//================= Main =========================================

int main (int argc, char *argv[]) {
	int i, argcount=0;
	string file_strings;
	Pair_database db;
	int db_size = 0;
	int voc_size = 0;
	int str_len = 0;
	int verbosity = 0;

	//To analize the input arguments
	for (i = 1; i < argc; i++) {

		if (strcmp ("-o", argv[i]) == 0) {
			file_strings = argv[++i];
			argcount++;
		} else if (strcmp("-dbs", argv[i]) == 0) {
			db_size = atoi(argv[++i]);
			argcount++;
		} else if (strcmp("-vs", argv[i]) == 0) {
			voc_size = atoi( argv[++i]);
			argcount++;
		} else if (strcmp("-sl", argv[i]) == 0) {
			str_len = atoi(argv[++i]);
			argcount++;
		} else if (strcmp ("-v", argv[i]) == 0) {
			verbosity = atoi(argv[++i]);
		} else if (strcmp ("-s", argv[i]) == 0) {
			srand48(atoi(argv[++i]));
		} else if (strcmp ("-H", argv[i]) == 0) {
			cout << "NAME" << endl;
			cout << "	strings : Make strings defined by vocabulary size and its length" << endl;
			cout << "SYNOPSIS" << endl;
			cout << "	string [OPTIONS] -dbs {output-size} -vs {vocabulary-size}" <<
				"-sl {string-length} -o {output-file}" << endl;
			cout << "DESCRIPTION" << endl;
			cout << "	Make output-size strings with a string length specified over" <<
				" a vocabulary size" << endl;
			cout << "	and save it into output-file.	" << endl;
			cout << "	Mandatory arguments:" << endl;
			cout << "	-vs vocabulary size from the <a> letter" << endl;
			cout << "	-sl length of the strings that will be generated" << endl;
			cout << "	-dbs number of generated strings" << endl;
			cout << "	-o output file where the strings will be saved" << endl;
			cout << "	Optional arguments" << endl;
			cout << "	-s seed to make the strings" << endl;
			cout << "	-v verbosity level" << endl;
			cout << "	-h brief help" << endl;
			cout << "	-H help" << endl;
			cout << "FILE FORMAT" << endl;
			cout << "	Output file: " << endl;
			cout << "	VocabularySize	StringLength" << endl;
			cout << "	OneStringByLine" << endl << endl;
			cout << "SEE ALSO" << endl;
			cout << "	m-strings" << endl;
			exit(0);
		} else if (strcmp ("-h", argv[i]) == 0) {
			cout << "use " << argv[0] << " [options]" << endl;
			cout << "Mandatory :" << endl;
			cout << "  -o   <string>   	String file" << endl;
			cout << "  -dbs <string>      	Data base size" << endl;
			cout << "  -vs  <int>		Vocabulary size" << endl;
			cout << "  -sl  <int>		String length" << endl;
			cout << "Optional :" << endl;
			cout << "  -s   <int> 		seed" << endl;
			cout << "  -v   <int> 		verbosity level" << endl;
			cout << "  -h			print this help" << endl;

			exit(0);
		} else {
			cout << "ERROR: unknown option " << argv[i] << endl;
			exit(-1);
		}
	}
	if (argcount < 4) {
		cout << "ERROR: many few options " << endl;
		cout << "use " << argv[0] << " [options]" << endl;
		cout << "Mandatory :" << endl;
		cout << "  -o   <string>   	String file" << endl;
		cout << "  -dbs <string>      	Data base size" << endl;
		cout << "  -vs  <int>		Vocabulary size" << endl;
		cout << "  -sl  <int>		String length" << endl;
		cout << "Optional :" << endl;
		cout << "  -s   <int> 		seed" << endl;
		cout << "  -v   <int> 		verbosity level" << endl;
		cout << "  -h			print this help" << endl;
		exit(-1);
	}

	MyString::initialize(voc_size);
	random_in_database(db_size, str_len, db);
	db.set_voc_size(voc_size);
	db.set_str_len(str_len);

	if( !file_strings.empty() ) {
		ofstream ds;
		ds.open(file_strings.c_str());
		ds << db;
		ds.close();
	}

	return 0;
}
