#include <iostream>
#include <fstream>
#include "trans-dist.h"

using namespace std;

//================= Main =========================================

int main (int argc, char *argv[]) {
	int i, argcount=0;
	string file_ftrans;
	string file_strans;
	int dbs = 0;
	int verbosity = 0;
	double (*distance)( Transducer&, Transducer&);

	distance = dist_conditional;


	//To analize the input arguments
	for (i = 1; i < argc; i++) {
		if (strcmp("-if", argv[i]) == 0) {
			argcount++;
			file_ftrans = argv[++i];
		} else if (strcmp("-is", argv[i]) == 0) {
			argcount++;
			file_strans = argv[++i];
		} else if (strcmp("-dbs", argv[i]) == 0) {
			dbs = atoi(argv[++i]);
		} else if( strcmp("-d", argv[i]) == 0 ) {
			i++;
			if( strcmp("c", argv[i]) == 0 ) {
				distance = dist_conditional;
			} else if( strcmp("j", argv[i]) == 0 ) {
				distance = dist_joint;
			} else {
				cout << "ERROR: "
				     << "unknown distance algorithm '"
				     << argv[i] 
				     << "'" << endl;
				exit(-1);
			}
		} else if (strcmp ("-v", argv[i]) == 0) {
			verbosity = atoi(argv[++i]);
		} else if (strcmp ("-H", argv[i]) == 0) {
			cout << "NAME" << endl;
			cout << "	trans-dist : Using this module you can compute the" <<
				" distance between two transducers" << endl;
			cout << "		     or costs model, using conditional or joint"<<
				" distance for this process." << endl;
			cout << "SYNOPSIS" << endl;
			cout << "	trans-dist -if {first-transducer} -is {second-transducer}" <<
				" -dbs {database-size} -o {output-file}" << endl;
			cout << "DESCRIPTION" << endl;
			cout << "	To compute the distance between two transducers." << endl;
			cout << "	Mandatory arguments:" << endl;
			cout << "	-if	First transducer model file" << endl;
			cout << "	-is	Second transducer model file" << endl;
			cout << "	Optional arguments" << endl;
			cout << "	-dbs	Size of the database used to learn" << endl;
			cout << "	-d {c,j}	Distance algorithm" << endl;
			cout << "			(default c:conditional)" << endl;
			cout << "	-v verbosity level" << endl;
			cout << "	-h brief help" << endl;
			cout << "	-H help" << endl;
			cout << "FILE FORMAT" << endl;
			cout << endl;
			cout << "	Transducer models: " << endl;
			cout << endl;
			cout << "	VocabularySize" << endl;
			cout << "	CostsMatrix" << endl;
			cout << endl;
			cout << "	Output: " << endl;
			cout << endl;
			cout << "	Distance	[DatabaseSize]" << endl;
			cout << endl;
			cout << "SEE ALSO" << endl;
			cout << "	___________" << endl;
			exit(0);
		} else if (strcmp ("-h", argv[i]) == 0) {
			cout << "use " << argv[0] << " [options]" << endl;
			cout << "Mandatory :" << endl;
			cout << "  -if   <string>	First transducer file" << endl;
			cout << "  -is   <string>	Second transducer file" << endl;
			cout << "Optional :" << endl;
			cout << "  -dbs  <int>		Experiment database size" << endl;
			cout << "  -d    {c,j}		Distance algorithm" << endl;
			cout << "			(Default c:conditional)" << endl;
			cout << "  -v    <int> 		Verbosity level" << endl;
			cout << "  -h 			Print brief help" << endl;
			cout << "  -H 			Print help" << endl;
			exit(0);
		} else {
			cout << "ERROR: unknown option " << argv[i] << endl;
			exit(-1);
		}
	}

	if (argcount < 2) {
		cout << "ERROR: many few options " << endl;
		cout << "use " << argv[0] << " [options]" << endl;
		cout << "Mandatory :" << endl;
		cout << "  -if   <string>	First transducer file" << endl;
		cout << "  -is   <string>	Second transducer file" << endl;
		cout << "Optional :" << endl;
		cout << "  -dbs  <int>		Experiment database size" << endl;
		cout << "  -d    {c,j}		Distance algorithm" << endl;
		cout << "			(Default c:conditional)" << endl;
		cout << "  -v    <int> 		Verbosity level" << endl;
		cout << "  -h 			Print brief help" << endl;
		cout << "  -H 			Print help" << endl;
		exit(-1);
	}

	Transducer t_ftrans = input_transducer(file_ftrans, verbosity);
	Transducer t_strans = input_transducer(file_strans, verbosity);

	print_distance(dbs, distance(t_ftrans,t_strans));

return 0;
}
