#ifndef distance_h
#define distance_h

#include <iostream>
#include <fstream>
#include <vector>
#include "math.h"
#include "../src/transducer.h"
#include "../src/pair_database.h"
#include "../src/trellis.h"

struct distance {

	distance() {
	}

};

Pair_database input_database(string file_db, int verbosity) {
	Pair_database db;
	if( !file_db.empty() ) {
		ifstream ds(file_db.c_str());
		if( !ds ) {
			cout << "ERROR: I can't open '" 
                             << file_db.c_str() << "'" << endl;
			exit(-1);
		}
		ds >> db;
	}
	if( verbosity > 3 )  {
		cout << "Strings base:" << endl;
		cout << db;
	}
	return db;
}

Transducer input_transducer(string file_transducer, int verbosity) {
	Transducer t;
	if( !file_transducer.empty() ) {
		ifstream dt(file_transducer.c_str());
		if( !dt ) {
			cout << "ERROR: I can't open '" 
	                     << file_transducer.c_str() << "'" << endl;
			exit(-1);
		}
		dt >> t;
	}
	if( verbosity > 0 ) {
		cout << "Result:" << endl;
		cout << t << endl;
	}
	return t;
}

double dist( const Transducer &t1, const Transducer &t2 ) {
	double acc = 0.0;

	if( t1.size() != t2.size() ) {
		cerr << "ERROR: I canot compute the distance" << endl;
		abort();
	}
	for( int i = 0; i < t1.size(); i++ )
		acc += pow( t1.ins(i) - t2.ins(i), 2.0 );
	for( int i = 0; i < t1.size(); i++ )
		acc += pow( t1.del(i) - t2.del(i), 2.0 );
	for( int i = 0; i < t1.size(); i++ )
		for( int j = 0; j < t1.size(); j++ )
			acc += pow( t1.sub(i,j) - t2.sub(i,j), 2.0 );
	acc += pow( t1.end() - t2.end(), 2.0 );
	return sqrt(acc);
}

double dist_conditional( Transducer &t1, Transducer &t2 ) {
	double a = 0.0;
	double b = 0.0;
	double dstar = 0.0;

	if( t1.size() != t2.size() ) {
		cerr << "ERROR: I cannot compute the distance" << endl;
		abort();
	}

	// To calculate A
	for( int i = 0; i < t1.size(); i++ )
		a += fabs( t2.ins(i) - t1.ins(i) );
	for( int i = 0; i < t1.size(); i++ )
		for( int j = 0; j < t1.size(); j++ ){
//			cout << "Value: " << t1.sub(i,j) << "--"<< t2.sub(i,j)<< endl;
			a += fabs( t1.sub(i,j) - t2.sub(i,j) );}
	// To calculate B
	for( int i = 0; i < t1.size(); i++ )
		b += fabs( t1.del(i) - t2.del(i) );
	b += fabs( t1.end() - t2.end() );

	dstar = (a + (b*t1.size())) / (2.0*t1.size());
//	cout << "Conditional a: " << a << " b: " << b << endl;
	return dstar;
}

double dist_joint( Transducer &t1, Transducer &t2 ) {
	double a = 0.0;
	double b = 0.0;
	double dstar = 0.0;

	if( t1.size() != t2.size() ) {
		cerr << "ERROR: I cannot compute the distance" << endl;
		abort();
	}

	// To calculate A
	for( int i = 0; i < t1.size(); i++ )
		a += fabs( t1.ins(i) - t2.ins(i));
	for( int i = 0; i < t1.size(); i++ )
		for( int j = 0; j < t1.size(); j++ )
			a += fabs( t1.sub(i,j) - t2.sub(i,j) );

	// To calculate B
	for( int i = 0; i < t1.size(); i++ )
		b += fabs( t1.del(i) - t2.del(i) );
	b += fabs( t1.end() - t2.end() );

	dstar = (a + b) / 2.0;
//	cout << "Join a: " << a << " b: " << b << endl;
	return dstar;
}

double log_prob( 
		const Transducer    &t, 
		const Pair_database &db ) {

	double acc;

	acc = 0.0;
	for( int i = 0; i < db.size(); i++ ) {
		Trellis beta = backward(t, db.in(i), db.out(i));
		acc += log( beta(0,0) * t.end());
	}
	return acc;
}

void print_distance(int dbs, double distance)
{
		cout << "Distance	Database size " << endl;
		cout << distance << "	" << dbs << endl;
}

#endif

